#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Script type: Python-fu
Script Title: Roller
Tested on GIMP version: 2.10.22; Windows 10, 64 bit; GTK 2.28.7

What is Does:
    Creates image compositions like wallpaper.

Where The Script Installs:
    See the how-to file.

Get the latest version: github.com/gummycowboy
"""
DEBUG = 0

if DEBUG:
    # Trace errors by creating a file if it doesn't already exist that
    # will accept and append error messages and piped (">>") print statements.
    import time
    import sys
    sys.stderr = open("D:\\roller_1_error.txt", 'a')
    print >> sys.stderr, "\nRoller,", time.ctime(), "\n"

from gimpfu import pdb
import gimpfu as fu
import os
import pygtk
import sys
pygtk.require("2.0")

# Append Roller paths to ‟sys.path” for GIMP's Python
# interpreter. ‟__file__” is GIMP's plug-in path. In the
# ‟roller” folder there are some un-categorized modules:
n = os.path.dirname(__file__) + os.path.sep + "roller"
q = [n]

# These are sub-‟roller” folders where there are more modules:
for i in ("effect", "preset", "style", "ui"):
    q.append(n + os.path.sep + i)

for i in q:
    # If a folder path isn't in ‟sys.path”, then add the path:
    if i not in sys.path:
        sys.path.append(i)

# The Python interpreter will now look for
# modules inside the various roller folders.
# So now it's okay to import from the roller
# folders inside the GIMP's plug-in folder.
# A limitation to this import method is that the modules are not
# identified by the interpreter as being part of a Python package.
from roller_base import Comm
from roller_stat import Stat
from roller_ui_main import UIMain


def start():
    """
    Start the program.

    typical variable usage:
        a : part of a sequence; object, int
        b : see ‟a”
        c : see ‟a”; column
        d : dict
        e : dict
        f : float
        g : widget; window
        h : height
        i : iteration
        j : RollerImage; GIMP image; iteration
        k : key
        m : flag
        n : string
        p : process
        q : iterable
        r : row
        s : size
        t : size
        u : point
        v : point
        w : width
        x : coordinate; index
        y : coordinate
        z : layer
    """

    # Save the interface context so that it can be restored:
    pdb.gimp_context_push()

    pdb.gimp_context_set_interpolation(fu.INTERPOLATION_CUBIC)
    pdb.gimp_context_set_antialias(1)
    pdb.gimp_context_set_sample_transparent(1)

    stat = Stat()

    try:
        UIMain(stat)

    except Exception as ex:
        Comm.show_err(ex)
        if DEBUG:
            print >> sys.stderr, ex

    # Return undo functionality:
    if stat.render:
        pdb.gimp_image_undo_group_end(stat.render)

    # Restore the interface context:
    pdb.gimp_context_pop()


fu.register(
    # name
    # becomes dialog title as python-fu + name
    # Space character is not allowed.
    # ‟name” is case-sensitive:
    "Roller",

    # tool-tip and window-tip text:
    "Renders image compositions.",

    # help (describe how-to, exceptions and dependencies).
    # This info will display in the plug-in browser:
    "Creates a new image and does not require an open image.",

    # The author is displayed in the plug-in browser:
    "Charles Bartley",

    # The copyright is displayed in the plug-in browser:
    "Charles Bartley",

    # The date is displayed in the plug-in browser:
    "2021",

    # menu item descriptor with short-cut key ‟_R”:
    "_Roller...",

    # image types
    # An empty string equates to no image:
    "",

    # dialog parameters:
    [],

    # results:
    [],

    # dialog function handler
    # The second item is the menu's location:
    start, menu="<Image>/Filters/Render")
if __name__ == "__main__":
    fu.main()
