#!/usr/bin/env python
# -*- coding: utf-8 -*-
from collections import OrderedDict
from copy import deepcopy
from roller_base import OZ
from roller_constant import (
        ForBackdropStyle,
        ForFill,
        ForFormat,
        ForWidget,
        FormatKey,
        LayoutKey,
        OptionKey,
        SessionKey
    )
import gtk


class Format:
    """Manage UIFormat session data."""
    fk = FormatKey

    # Read only:
    default = {
            fk.NAME: "Default Format",
            fk.EFFECT: ForFormat.NONE,
            fk.SHOW_IN_LAYOUT: True,
            fk.RESIZE: ForFormat.LOCKED,
            fk.HORIZONTAL: ForFormat.CENTER,
            fk.VERTICAL: ForFormat.MIDDLE,
            fk.IMAGE: ForFormat.NEXT,
            fk.ROTATE: 0,
            fk.OPACITY: 100,
            fk.BLUR_BEHIND: 0,
            fk.FLIP_HORIZONTAL: 0,
            fk.FLIP_VERTICAL: 0,
            fk.MERGE_PER_CELL: 0,
            fk.PROPERTY_PER_CELL: 0,
            fk.PLACEMENT_PER_CELL: 0,
            fk.FIXED_MARGIN_PER_CELL: 0,
            fk.PERCENT_MARGIN_PER_CELL: 0,
            fk.ROW: 1,
            fk.COLUMN: 1,
            fk.LAYER_MARGIN_TOP_FIXED: 0,
            fk.LAYER_MARGIN_LEFT_FIXED: 0,
            fk.LAYER_MARGIN_RIGHT_FIXED: 0,
            fk.LAYER_MARGIN_BOTTOM_FIXED: 0,
            fk.LAYER_MARGIN_TOP_PERCENT: 0,
            fk.LAYER_MARGIN_LEFT_PERCENT: 0,
            fk.LAYER_MARGIN_RIGHT_PERCENT: 0,
            fk.LAYER_MARGIN_BOTTOM_PERCENT: 0,
            fk.CELL_MARGIN_TOP_FIXED: 0,
            fk.CELL_MARGIN_LEFT_FIXED: 0,
            fk.CELL_MARGIN_RIGHT_FIXED: 0,
            fk.CELL_MARGIN_BOTTOM_FIXED: 0,
            fk.CELL_MARGIN_TOP_PERCENT: 0,
            fk.CELL_MARGIN_LEFT_PERCENT: 0,
            fk.CELL_MARGIN_RIGHT_PERCENT: 0,
            fk.CELL_MARGIN_BOTTOM_PERCENT: 0,
            fk.PRESET: "Default",
            fk.CELL_TABLE_MERGE: None,
            fk.CELL_TABLE_PLACEMENT: None,
            fk.CELL_TABLE_FIXED_MARGIN: None,
            fk.CELL_TABLE_PERCENT_MARGIN: None,
            fk.CELL_TABLE_PROPERTY: None
        }


class Session:
    """
    Store session control values.

    After a successful render, a "Last Used" session file is
    written if the current session dictionary differs from
    the last used dictionary at the start of program.
    """
    w, h = gtk.gdk.screen_width(), gtk.gdk.screen_height()
    a, sk, ok = LayoutKey, SessionKey, OptionKey

    # read-only:
    default = {
        sk.AVERAGE_COLOR: OrderedDict([
                (ok.MODE, "Normal"),
                (ok.OPACITY, 100.),
                (ok.INVERT, 0)
            ]),

        sk.AUTOMATE: 0,
        sk.BACKDROP_IMAGE: OrderedDict([
                (ok.BACKDROP_IMAGE, ForWidget.FIRST),
                (ok.BACKDROP_BLUR, 0),
                (ok.FIT_IMAGE, 1),
                (ok.INVERT, 0)
            ]),

        sk.BACKDROP_STYLE: ForBackdropStyle.TRANSPARENCY,
        sk.BORDER_GRADIENT: OrderedDict([
                (ok.GRADIENT, "Default"),
                (ok.GRADIENT_TYPE, "Bilinear"),
                (ok.OPACITY, 100),
                (ok.ROTATE, 0),
                (ok.INVERT, 0),
                (ok.REVERSE, 0),
                (ok.OFFSET, 0),
                (ok.START_X, 0),
                (ok.START_Y, 0),
                (ok.END_X, w - 1),
                (ok.END_Y, h - 1)
            ]),

        sk.BORDER_LINE: OrderedDict([
                (ok.BORDER_WIDTH, 10),
                (ok.LIGHT_ANGLE, 45.),
                (ok.BORDER_TYPE, 1),
                (ok.INHERIT_OPACITY, 0),
                (ok.RANDOM_SEED, 0),
                (ok.POWER, 30)
            ]),

        sk.CERAMIC_CHIP: OrderedDict([
                (ok.WIDTH, 45),
                (ok.BORDER_WIDTH, 8),
                (ok.MESH_TYPE, "Triangle"),
                (ok.COLOR_1, (0, 255, 0)),
                (ok.LIGHT_ANGLE, 45.),
                (ok.BORDER_TYPE, 1),
                (ok.INHERIT_OPACITY, 0),
                (ok.RANDOM_SEED, 0),
                (ok.POWER, 30)
            ]),

        sk.CIRCLE_PUNCH: OrderedDict([
                (ok.CIRCLE_DIAMETER, 36),
                (ok.WIDTH, 45),
                (ok.ROTATE, 0),
                (ok.BORDER_WIDTH, 10),
                (ok.LIGHT_ANGLE, 45.),
                (ok.BORDER_TYPE, 1),
                (ok.INHERIT_OPACITY, 0),
                (ok.RANDOM_SEED, 0),
                (ok.POWER, 30)
            ]),

        sk.CLEAR_FRAME: OrderedDict([
                (ok.BORDER_WIDTH, 45),
                (ok.BORDER_TYPE, 1),
                (ok.COLOR_1, (255, 255, 255))
            ]),

        sk.COLOR_FILL: OrderedDict([
                (ok.THRESHOLD, 1.),
                (ok.CRITERIA, ForFill.CRITERIA_LIST[0]),
                (ok.COLOR_1, (127, 127, 127)),
                (ok.MODE, "Normal"),
                (ok.OPACITY, 100),
                (ok.INVERT, 0),
                (ok.START_X, 0),
                (ok.START_Y, 0)
            ]),

        sk.COLORED_BOARD: OrderedDict([
                (ok.BORDER_WIDTH, 1),
                (ok.BORDER_TYPE, 1),
                (ok.COLOR_1, (255, 255, 255)),
                (ok.INHERIT_OPACITY, 0)
            ]),

        sk.COLORED_GRID: OrderedDict([
                (ok.ROW, 4),
                (ok.COLUMN, 4),
                (ok.COLOR_1, (0,) * 3),
                (ok.COLOR_2, (255, 255, 255)),
                (ok.MODE, "Normal"),
                (ok.OPACITY, 100),
                (ok.ROTATE, 0),
                (ok.INVERT, 0)
            ]),

        sk.CORE_DESIGN: OrderedDict([
                (ok.GRADIENT, "Default"),
                (ok.ROW, 12),
                (ok.COLUMN, 12),
                (ok.INVERT, 0),
                (ok.REVERSE, 0),
                (ok.OFFSET, 0)
            ]),

        sk.DROP_SHADOW: OrderedDict([
                (ok.SHADOW_BLUR, 30),
                (ok.INTENSITY, 120),
                (ok.OFFSET_X, 0),
                (ok.OFFSET_Y, 0),
                (ok.SHADOW_COLOR, (0, 0, 0)),
                (ok.INHERIT_OPACITY, 1)
            ]),

        sk.FILL_LIGHT_SHADOW: OrderedDict([
                (ok.SHADOW_BLUR, 30),
                (ok.INTENSITY, 120),
                (ok.OFFSET_X, 0),
                (ok.OFFSET_Y, 0),
                (ok.SHADOW_COLOR, (0, 0, 0)),
                (ok.INHERIT_OPACITY, 0)
            ]),

        sk.FLOOR_SAMPLE: OrderedDict([
                (ok.ANGLE, 60),
                (ok.COLOR_1, (255, 255, 255)),
                (ok.COLOR_2, (0, 0, 0)),
                (ok.INVERT, 0)
            ]),

        sk.FORMAT_LIST: [],
        sk.GLASS_GAW: {ok.RANDOM_SEED: 0},
        sk.GRADIENT_BEVEL: OrderedDict([
                (ok.BORDER_WIDTH, 40),
                (ok.COLOR_1, (0, 0, 0)),
                (ok.COLOR_2, (255, 255, 255)),
            ]),

        sk.GRADIENT_FILL: OrderedDict([
                (ok.GRADIENT, "Default"),
                (ok.GRADIENT_TYPE, "Linear"),
                (ok.ROTATE, 0),
                (ok.MODE, "Normal"),
                (ok.OPACITY, 100),
                (ok.INVERT, 0),
                (ok.REVERSE, 0),
                (ok.OFFSET, 0),
                (ok.START_X, 0),
                (ok.START_Y, 0),
                (ok.END_X, w - 1),
                (ok.END_Y, h - 1)
            ]),

        sk.HEIGHT: h,
        sk.HONEY_BEE:  OrderedDict([
                (ok.MESH_SIZE, 50),
                (ok.MESH_TYPE, "Hexagon"),
                (ok.NEATNESS, 1.),
                (ok.RANDOM_SEED, 0)
            ]),

        sk.IMAGE_EDGE_SHADOW: OrderedDict([
                (ok.INLAY_BLUR, 20),
                (ok.INTENSITY, 120),
                (ok.SHADOW_COLOR, (0, 0, 0))
            ]),

        sk.IMAGE_GRADIENT: OrderedDict([
                (ok.NAME, "Sampled Gradient"),
                (ok.SAMPLE_RADIUS, 1),
                (ok.SAMPLE_POINTS, 2),
                (ok.SAMPLE_VECTOR, "Mid-Vertical"),
                (ok.KEEP_GRADIENT, 0),
                (ok.GRADIENT_TYPE, "Linear"),
                (ok.ROTATE, 0),
                (ok.MODE, "Normal"),
                (ok.OPACITY, 100),
                (ok.INVERT, 0),
                (ok.REVERSE, 0)
            ]),

        sk.INLAY_SHADOW: OrderedDict([
                (ok.INLAY_BLUR, 30),
                (ok.INTENSITY, 100),
                (ok.SHADOW_COLOR, (0, 0, 0))
            ]),

        sk.JAGGED_EDGE: OrderedDict([
                (ok.AMPLITUDE, 3),
                (ok.SMOOTHNESS, 4),
                (ok.SHADOW, SessionKey.DROP_SHADOW),
                (ok.RANDOM_SEED, 0)
            ]),

        sk.KEY_LIGHT_SHADOW: OrderedDict([
                (ok.SHADOW_BLUR, 30),
                (ok.INTENSITY, 120),
                (ok.OFFSET_X, 0),
                (ok.OFFSET_Y, 0),
                (ok.SHADOW_COLOR, (0, 0, 0)),
                (ok.INHERIT_OPACITY, 0)
            ]),

        sk.LAYOUT_OPTION: OrderedDict([
            (a.CELL_MARGINS, 0),
            (a.COORDINATES, 0),
            (a.CORNERS, 0),
            (a.DIMENSIONS, 0),
            (a.GRID, 0),
            (a.LAYER_MARGINS, 0),
            (a.NAME, 0),
            (a.RATIOS, 0),
        ]),

        sk.LIGHT_SHAFT: OrderedDict([
                (ok.ROTATE, 0),
                (ok.SCALE, 7),
            ]),

        sk.LINE_FASHION: OrderedDict([
                (ok.WIDTH, 45),
                (ok.LINE_WIDTH, 12),
                (ok.GAP_WIDTH, 12),
                (ok.BORDER_WIDTH, 10),
                (ok.ROTATE, 0),
                (ok.LIGHT_ANGLE, 45.),
                (ok.BORDER_TYPE, 1),
                (ok.INHERIT_OPACITY, 0),
                (ok.RANDOM_SEED, 0),
                (ok.POWER, 30)
            ]),

        sk.LOST_MAZE: OrderedDict([
                (ok.ROW, 10),
                (ok.COLUMN, 10),
                (ok.GRADIENT, "Default"),
                (ok.INVERT, 0),
                (ok.REVERSE, 0),
                (ok.OFFSET, 0),
                (ok.RANDOM_SEED, 0)
            ]),

        sk.MAZE_MIRROR: OrderedDict([
                (ok.ROW, 10),
                (ok.COLUMN, 10),
                (ok.MAZE_TYPE, ForBackdropStyle.SCATTERED_CONNECTORS),
                (ok.DIRECTION, ForBackdropStyle.CLOCKWISE),
                (ok.CELL_GAP, 3),
                (ok.GAP_TYPE, ForBackdropStyle.RANDOM),
                (ok.STOP_LENGTH, 25),
                (ok.SCATTER_COUNT, 4),
                (ok.BORDER_WIDTH, 8),
                (ok.COMPOSITION_BORDER_WIDTH, 8),
                (ok.LINE_WIDTH, 3),
                (ok.LIGHT_ANGLE, 45.),
                (ok.BORDER_TYPE, 1),
                (ok.INHERIT_OPACITY, 0),
                (ok.RANDOM_SEED, 0),
                (ok.POWER, 30)
            ]),

        sk.MYSTERY_GRATE: OrderedDict([
                (ok.GRADIENT, "Default"),
                (ok.COLUMN_1, 8),
                (ok.COLUMN_2, 80),
                (ok.INVERT, 0),
                (ok.REVERSE, 0),
                (ok.OFFSET, 0)
            ]),

        sk.PATTERN_FILL: OrderedDict([
                (ok.PATTERN, None),
                (ok.CRITERIA, ForFill.CRITERIA_LIST[0]),
                (ok.THRESHOLD, 1.),
                (ok.MODE, "Normal"),
                (ok.OPACITY, 100),
                (ok.INVERT, 0),
                (ok.START_X, 0),
                (ok.START_Y, 0)
            ]),

        sk.PRESET: "Default",
        sk.RAD_WAVE: OrderedDict([
                (ok.BORDER_WIDTH, 10),
                (ok.LINE_WIDTH, 8),
                (ok.COMPOSITION_BORDER_WIDTH, 8),
                (ok.ROW, 2),
                (ok.COLUMN, 2),
                (ok.WAVE_AMPLITUDE, 10),
                (ok.WAVELENGTH, 25),
                (ok.WHIRL, 0),
                (ok.LIGHT_ANGLE, 45.),
                (ok.BORDER_TYPE, 1),
                (ok.INHERIT_OPACITY, 0),
                (ok.RANDOM_SEED, 0),
                (ok.POWER, 30)
            ]),

        sk.RAINBOW_VALLEY: OrderedDict([
                (ok.TEXTURE, 0),
                (ok.EMBOSS, 0),
                (ok.RANDOM_SEED, 0),
                (ok.POWER, 30)
            ]),

        sk.ROCKY_LANDING:  OrderedDict([
                (ok.BLEND, 12),
                (ok.RANDOM_SEED, 0)
            ]),

        sk.ROUNDED_EDGE: OrderedDict([
                (ok.ROUNDED_EDGE_BLUR, 15),
                (ok.ROUND_UP, 0),
            ]),

        sk.SPACETIME_FABRIC: OrderedDict([
                (ok.ROW, 10),
                (ok.COLUMN, 10),
                (ok.COMPONENT, "Green"),
                (ok.EMBOSS, 0)
            ]),

        sk.STAINED_GLASS: OrderedDict([
                (ok.WIDTH, 45),
                (ok.PANE_WIDTH, 50),
                (ok.PANE_HEIGHT, 50),
                (ok.ROTATE, 45),
                (ok.BORDER_WIDTH, 8),
                (ok.LIGHT_ANGLE, 45.),
                (ok.BORDER_TYPE, 1),
                (ok.INHERIT_OPACITY, 0),
                (ok.RANDOM_SEED, 0),
                (ok.POWER, 30)
            ]),

        sk.SQUARE_PUNCH: OrderedDict([
                (ok.WIDTH, 45),
                (ok.LINE_WIDTH, 12),
                (ok.GAP_WIDTH, 12),
                (ok.BORDER_WIDTH, 10),
                (ok.ROTATE, 0),
                (ok.LIGHT_ANGLE, 45.),
                (ok.BORDER_TYPE, 1),
                (ok.INHERIT_OPACITY, 0),
                (ok.RANDOM_SEED, 0),
                (ok.POWER, 30)
            ]),

        sk.WIDTH: w,
        sk.WIRE_FENCE: OrderedDict([
                (ok.MESH_TYPE, "Hexagon"),
                (ok.MESH_SIZE, 30),
                (ok.WIDTH, 35),
                (ok.WIRE_THICKNESS, 3),
                (ok.NEATNESS, 1.),
                (ok.BORDER_WIDTH, 8),
                (ok.LIGHT_ANGLE, 45.),
                (ok.BORDER_TYPE, 1),
                (ok.INHERIT_OPACITY, 0),
                (ok.RANDOM_SEED, 0),
                (ok.POWER, 30)
            ]),
        }

    @staticmethod
    def load_file(d):
        """
        Load a Session file.
        Pass it through a validity check.

        d: dict
            pickle dict

        Return the Session data or (0 or -1) for failure.
        """
        e = OZ.pickle_load(d)

        if e:
            d1 = deepcopy(Session.default)

            OZ.pass_version(e, d1)
            for d1 in e[SessionKey.FORMAT_LIST]:
                OZ.pass_version(d1, Format.default, is_format=1)

        else:
            e = 0
        return e
