#!/usr/bin/env python
# -*- coding: utf-8 -*-
from gimpfu import pdb
from random import choice, randint
from roller_base import Base
from roller_constant import (
        ForBackdropStyle,
        ForCell,
        ForEffect,
        ForFill,
        ForGradient,
        OptionKey
    )

from roller_mode import Mode
from roller_check_button import RollerCheckButton
from roller_color_button import RollerColorButton
from roller_combobox import RollerComboBox
from roller_entry import RollerEntry
from roller_layout import RollerImage
from roller_radio_button import RollerRadioButton
from roller_spin_button import RollerSpinButton


class OptionLimit:
    """Use to define a limited range and a random range for option values."""

    def __init__(self, stat):
        """
        stat: Stat
            global variables
        """
        self.stat = stat
        a = OptionLimitKey
        fol = ForOptionLimit
        ok = OptionKey
        self.delimiter = {
            ok.AMPLITUDE: {
                    a.RANGE: (2, 10),
                    a.LIMIT: (2, 10),
                    a.WIDGET: RollerSpinButton
                },

            ok.ANGLE: {
                    a.RANGE: (6, 90),
                    a.LIMIT: (30, 90),
                    a.WIDGET: RollerSpinButton
                },

            ok.BACKDROP_BLUR: {
                    a.RANGE: (0, 1000),
                    a.LIMIT: (0, 200),
                    a.WIDGET: RollerSpinButton
                },

            ok.BACKDROP_IMAGE: {
                    a.RANGE_FUNCTION: self.get_image_list,
                    a.WIDGET: RollerComboBox
                },

            ok.BLEND: {
                    a.RANGE: (1, 60),
                    a.LIMIT: (5, 20),
                    a.WIDGET: RollerSpinButton
                },

            ok.BORDER_TYPE: {
                    a.LIMIT: (0, 1),
                    a.LABEL:  ("Rounded", "Angular"),
                    a.WIDGET: RollerRadioButton,
                    a.CATEGORY: fol.DESIGN
                },

            ok.BORDER_WIDTH: {
                    a.RANGE_FUNCTION: self.get_max_span,
                    a.LIMIT_SELF: self.get_border_width,
                    a.WIDGET: RollerSpinButton,
                    a.CATEGORY: fol.SCALE
                },

            ok.CELL_GAP: {
                    a.RANGE_FUNCTION: self.get_gap_range,
                    a.LIMIT: (2, 12),
                    a.WIDGET: RollerSpinButton,
                    a.CATEGORY: fol.DESIGN
                },

            ok.CIRCLE_DIAMETER: {
                    a.RANGE_FUNCTION: self.get_circle_span,
                    a.LIMIT_SELF: self.get_filler_width,
                    a.WIDGET: RollerSpinButton
                },

            ok.COLOR_1: {
                    a.LIMIT_FUNCTION: Base.rnd_col,
                    a.WIDGET: RollerColorButton
                },

            ok.COLOR_2: {
                    a.LIMIT_FUNCTION: Base.rnd_col,
                    a.WIDGET: RollerColorButton
                },

            ok.COLUMN: {
                    a.RANGE_SELF: self.get_column_range,
                    a.LIMIT_SELF: self.get_column_limited,
                    a.WIDGET: RollerSpinButton,
                    a.CATEGORY: fol.DESIGN
                },

            ok.COLUMN_1: {
                    a.RANGE_SELF: self.get_column_range,
                    a.LIMIT_SELF: self.get_column,
                    a.WIDGET: RollerSpinButton
                },

            ok.COLUMN_2: {
                    a.RANGE_SELF: self.get_column_range,
                    a.LIMIT_SELF: self.get_column,
                    a.WIDGET: RollerSpinButton
                },

            ok.COMPONENT: {
                    a.LIST: ForBackdropStyle.COMPONENT,
                    a.WIDGET: RollerComboBox
                },

            ok.COMPOSITION_BORDER_WIDTH: {
                    a.RANGE_FUNCTION: self.get_max_span,
                    a.LIMIT_SELF: self.get_border_width,
                    a.WIDGET: RollerSpinButton,
                    a.CATEGORY: fol.SCALE
                },

            ok.CRITERIA: {
                    a.LIST: ForFill.CRITERIA_LIST,
                    a.WIDGET: RollerComboBox
                },

            ok.DIRECTION: {
                    a.LIST: ForBackdropStyle.DIRECTION,
                    a.WIDGET: RollerComboBox,
                    a.CATEGORY: fol.DESIGN
                },

            ok.EMBOSS: {
                    a.LIMIT: (0, 1),
                    a.WIDGET: RollerCheckButton
                },

            ok.END_X: {
                    a.RANGE_FUNCTION: self.get_coordinate_x_range,
                    a.LIMIT_FUNCTION: self.get_coordinate_x,
                    a.WIDGET: RollerSpinButton
                },

            ok.END_Y: {
                    a.RANGE_FUNCTION: self.get_coordinate_y_range,
                    a.LIMIT_FUNCTION: self.get_coordinate_y,
                    a.WIDGET: RollerSpinButton
                },

            ok.FIT_IMAGE: {a.WIDGET: RollerCheckButton},
            ok.GAP_TYPE: {
                    a.LIST: ForBackdropStyle.GAP_TYPE,
                    a.WIDGET: RollerComboBox,
                    a.CATEGORY: fol.DESIGN
                },

            ok.GAP_WIDTH: {
                    a.RANGE_FUNCTION: self.get_max_span,
                    a.LIMIT_SELF: self.get_border_width,
                    a.WIDGET: RollerSpinButton
                },

            ok.GRADIENT: {
                    a.FUNCTION: self.get_gradient_list,
                    a.WIDGET: RollerComboBox
                },

            ok.GRADIENT_TYPE: {
                    a.LIST: ForGradient.GRADIENT_TYPE,
                    a.WIDGET: RollerComboBox
                },

            ok.INHERIT_OPACITY: {
                    a.LABEL: ("Same as image", "Make opaque"),
                    a.WIDGET: RollerRadioButton,
                    a.CATEGORY: fol.DESIGN
                },

            ok.INLAY_BLUR: {
                    a.RANGE: (0, 1000),
                    a.LIMIT: (20, 30),
                    a.WIDGET: RollerSpinButton
                },

            ok.INVERT: {
                    a.LIMIT: (0, 1),
                    a.WIDGET: RollerCheckButton
                },

            ok.INTENSITY: {
                    a.RANGE: (0, 1000),
                    a.LIMIT: (80, 130),
                    a.WIDGET: RollerSpinButton
                },

            ok.KEEP_GRADIENT: {a.WIDGET: RollerCheckButton},
            ok.LIGHT_ANGLE: {
                    a.RANGE: (0, 359),
                    a.LIMIT: (0, 359),
                    a.WIDGET: RollerSpinButton,
                    a.CATEGORY: fol.DESIGN
                },

            ok.LINE_WIDTH: {
                    a.RANGE_FUNCTION: self.get_max_span,
                    a.LIMIT_SELF: self.get_border_width,
                    a.WIDGET: RollerSpinButton,
                    a.CATEGORY: fol.SCALE
                },

            ok.MAZE_TYPE: {
                    a.LIST: ForBackdropStyle.MAZE_TYPE,
                    a.WIDGET: RollerComboBox,
                    a.CATEGORY: fol.DESIGN
                },

            ok.MESH_SIZE: {
                    a.RANGE: (2, 1000),
                    a.LIMIT: (25, 60),
                    a.WIDGET: RollerSpinButton
                },

            ok.MESH_TYPE: {
                    a.LIST: ForBackdropStyle.MESH_TYPE,
                    a.WIDGET: RollerComboBox
                },

            ok.MODE: {
                    a.RANGE_FUNCTION: Mode.names,
                    a.LIMIT_FUNCTION: Mode.rand,
                    a.WIDGET: RollerComboBox
                },

            ok.NAME: {a.WIDGET: RollerEntry},
            ok.NEATNESS: {
                    a.RANGE: (0., 1.),
                    a.LIMIT: (0., 1.),
                    a.STEP: (.01, .1),
                    a.WIDGET: RollerSpinButton
                },

            ok.OFFSET: {
                    a.RANGE_FUNCTION: self.get_max_span,
                    a.LIMIT_FUNCTION: self.get_gradient_offset_limited,
                    a.WIDGET: RollerSpinButton
                },

            ok.OFFSET_X: {
                    a.RANGE_FUNCTION: self.get_max_shadow_span,
                    a.LIMIT_FUNCTION: self.get_shadow_offset_limited,
                    a.WIDGET: RollerSpinButton
                },

            ok.OFFSET_Y: {
                    a.RANGE_FUNCTION: self.get_max_shadow_span,
                    a.LIMIT_FUNCTION: self.get_shadow_offset_limited,
                    a.WIDGET: RollerSpinButton
                },

            ok.OPACITY: {
                    a.RANGE: (0, 100),
                    a.LIMIT: (50, 100),
                    a.WIDGET: RollerSpinButton
                },

            ok.PANE_HEIGHT: {
                    a.RANGE_FUNCTION: self.get_pane_range,
                    a.LIMIT_FUNCTION: self.get_pane_limit,
                    a.WIDGET: RollerSpinButton
                },

            ok.PANE_WIDTH: {
                    a.RANGE_FUNCTION: self.get_pane_range,
                    a.LIMIT_FUNCTION: self.get_pane_limit,
                    a.WIDGET: RollerSpinButton
                },

            ok.PATTERN: {
                    a.FUNCTION: self.get_pattern_list,
                    a.WIDGET: RollerComboBox
                },

            ok.POWER: {
                    a.RANGE: (0, 180),
                    a.LIMIT: (0, 180),
                    a.WIDGET: RollerSpinButton,
                    a.CATEGORY: fol.RANDOM
                },

            ok.RANDOM_SEED: {
                    a.RANGE: (0, 2147483640),
                    a.LIMIT: (0, 2147483640),
                    a.WIDGET: RollerSpinButton,
                    a.CATEGORY: fol.RANDOM
                },

            ok.REVERSE: {
                    a.LIMIT: (0, 1),
                    a.WIDGET: RollerCheckButton
                },

            ok.ROTATE: {
                    a.RANGE: (-359, 359),
                    a.LIMIT: (-359, 359),
                    a.WIDGET: RollerSpinButton
                },

            ok.ROUND_UP: {
                    a.LIMIT: (0, 1),
                    a.WIDGET: RollerCheckButton
                },

            ok.ROUNDED_EDGE_BLUR: {
                    a.RANGE: (0, 1000),
                    a.LIMIT: (15, 30),
                    a.WIDGET: RollerSpinButton
                },

            ok.ROW: {
                    a.RANGE_SELF: self.get_row_range,
                    a.LIMIT_SELF: self.get_row_limited,
                    a.WIDGET: RollerSpinButton,
                    a.CATEGORY: fol.DESIGN
                },

            ok.SAMPLE_POINTS: {
                    a.RANGE: (2, 50),
                    a.LIMIT: (2, 25),
                    a.WIDGET: RollerSpinButton
                },

            ok.SAMPLE_RADIUS: {
                    a.RANGE: (1, 50),
                    a.LIMIT: (1, 15),
                    a.WIDGET: RollerSpinButton
                },

            ok.SAMPLE_VECTOR: {
                    a.LIST: ForGradient.VECTOR,
                    a.WIDGET: RollerComboBox
                },

            ok.SCALE: {
                    a.RANGE: (4, 20),
                    a.LIMIT: (4, 20),
                    a.WIDGET: RollerSpinButton
                },

            ok.SCATTER_COUNT: {
                    a.RANGE: (1, 30),
                    a.LIMIT: (2, 20),
                    a.WIDGET: RollerSpinButton,
                    a.CATEGORY: fol.DESIGN
                },

            ok.SHADOW: {
                    a.LIST: ForEffect.SHADOW_CHOICE,
                    a.WIDGET: RollerComboBox
                },

            ok.SHADOW_BLUR: {
                    a.RANGE: (0, 1000),
                    a.LIMIT: (20, 40),
                    a.WIDGET: RollerSpinButton
                },

            ok.SHADOW_COLOR: {a.WIDGET: RollerColorButton},
            ok.SMOOTHNESS: {
                    a.RANGE: (3, 20),
                    a.LIMIT: (5, 12),
                    a.WIDGET: RollerSpinButton
                },

            ok.START_X: {
                    a.RANGE_FUNCTION: self.get_coordinate_x_range,
                    a.LIMIT_FUNCTION: self.get_coordinate_x,
                    a.WIDGET: RollerSpinButton
                },

            ok.START_Y: {
                    a.RANGE_FUNCTION: self.get_coordinate_y_range,
                    a.LIMIT_FUNCTION: self.get_coordinate_y,
                    a.WIDGET: RollerSpinButton
                },

            ok.STOP_LENGTH: {
                    a.RANGE: (1, 100),
                    a.LIMIT: (20, 30),
                    a.WIDGET: RollerSpinButton,
                    a.CATEGORY: fol.DESIGN
                },

            ok.TEXTURE: {
                    a.LIMIT: (0, 1),
                    a.WIDGET: RollerCheckButton
                },

            ok.THRESHOLD: {
                    a.RANGE: (.0, 1.),
                    a.LIMIT: (.5, 1.),
                    a.STEP: (.01, .1),
                    a.WIDGET: RollerSpinButton
                },

            ok.WAVE_AMPLITUDE: {
                    a.RANGE: (0, 100),
                    a.LIMIT: (0, 10),
                    a.WIDGET: RollerSpinButton
                },

            ok.WAVELENGTH: {
                    a.RANGE: (.01, 50.),
                    a.LIMIT: (30., 50.),
                    a.STEP: (1., 1.),
                    a.WIDGET: RollerSpinButton
                },

            ok.WHIRL: {
                    a.RANGE: (-720, 720),
                    a.LIMIT: (-30, 30),
                    a.WIDGET: RollerSpinButton
                },
            ok.WIDTH: {
                    a.RANGE_FUNCTION: self.get_max_span,
                    a.LIMIT_SELF: self.get_filler_width,
                    a.WIDGET: RollerSpinButton
                },

            ok.WIRE_THICKNESS: {
                    a.RANGE: (3, 30),
                    a.LIMIT: (4, 10),
                    a.WIDGET: RollerSpinButton
                },
        }

    def get_border_width(self, caller):
        """
        Return a random value for a border width.

        Use a border width template.
            width_low: int
                low value of random range

            width_high: int
                high value of random range
        """
        return randint(caller.width_low, caller.width_high)

    def get_circle_span(self):
        """Return a maximum value for a dimension."""
        return (20, max(self.stat.width, self.stat.height))

    def get_column(self, caller):
        """
        Return a randomized column value.

        Use a RenderHub class template.
            random_rc: function
                Call to return a randomized column value.

        caller: RenderHub class instance
        """
        _, c = caller.random_rc()
        return c

    def get_column_limited(self, caller):
        """
        Return a self-limited randomized column value.

        Use a RenderHub class template.
            random_rc: function
                Call to return a randomized column value.

        caller: RenderHub class instance
        """
        _, c = caller.random_rc()
        return max(c, caller.column)

    def get_column_range(self, caller):
        """
        Return a column range.

        Use a RenderHub class template.
            random_rc: function
                Call to return a randomized column value.

            column: int
                Use to set the minimum column value.

        caller: RenderHub class instance
        """
        c = self.stat.height / ForCell.MIN_CELL_SPAN
        return (caller.column, max(caller.column, c))

    def get_coordinate_x(self):
        """Return a randomized fill-operation's x-coordinate."""
        return randint(0, self.stat.width - 1)

    def get_coordinate_x_range(self):
        """Return the valid range for a fill-operation's x coordinate."""
        return 0, self.stat.width - 1

    def get_coordinate_y(self):
        """Return a randomized fill-operation's y-coordinate."""
        return randint(0, self.stat.height - 1)

    def get_coordinate_y_range(self):
        """Return the valid range for a fill-operation's y-coordinate."""
        return 0, self.stat.height - 1

    def get_filler_width(self, caller):
        """
        Return a random value for a filler width.

        Use a border width template.
            filler_width_low: int
                low value of random range

            filler_width_high: int
                high value of random range
        """
        return randint(caller.filler_width_low, caller.filler_width_high)

    def get_gap_range(self):
        """Return the valid range for a maze gap."""
        a = self.stat.width / 2 * self.stat.height / 2
        return 1, a - 1

    def get_gradient_list(self):
        """Return a list of available gradients."""
        return sorted(pdb.gimp_gradients_get_list(None)[1])

    def get_gradient_offset_limited(self):
        """Return a tuple for a shadow offset."""
        return randint(0, min(self.stat.width, self.stat.height, 100))

    def get_image_list(self):
        """Return a backdrop image list."""
        return RollerImage.image_list[1:]

    def get_max_shadow_span(self):
        """Return a maximum value for a shadow offset."""
        a = max(self.stat.width, self.stat.height)
        return -a, a

    def get_max_span(self):
        """Return a maximum value for a dimension."""
        return (0, max(self.stat.width, self.stat.height))

    def get_pane_range(self):
        """Return a tuple for a glass pane range."""
        a = min(self.stat.width, self.stat.height)
        return min(25, a), a

    def get_pane_limit(self):
        """Return an integer for a randomized glass pane width."""
        a = min(self.stat.width, self.stat.height)
        return randint(min(25, a), min(300, a))

    def get_pattern_list(self):
        """Return a list of available patterns."""
        return pdb.gimp_patterns_get_list(None)[1]

    def get_row_range(self, caller):
        """Return a tuple for the range of available rows."""
        r = self.stat.width / ForCell.MIN_CELL_SPAN
        return (caller.row, max(r, caller.row))

    def get_row_limited(self, caller):
        """Return a randomized integer for a self-limited row value."""
        r, _ = caller.random_rc()
        return max(r, caller.row)

    def get_shadow_offset_limited(self):
        """Return a tuple for a shadow offset."""
        return randint(0, min(self.stat.radius, 10)) * choice((-1, 1))


class OptionLimitKey:
    """Has keys used to access the OptionLimit delimiter dictionary."""
    CATEGORY = 'category'
    FUNCTION = 'function'
    FUNCTION_RANGE = 'function_range'
    LIMIT_SELF = 'limit_self'
    LABEL = 'label'
    LIMIT = 'limit'
    LIMIT_FUNCTION = 'limit_function'
    LIST = 'list'
    RANGE = 'range'
    RANGE_FUNCTION = 'range_function'
    RANGE_SELF = 'range_self'
    STEP = 'step'
    WIDGET = 'widget'


class ForOptionLimit:
    """Has values used in the OptionLimit class."""
    DESIGN = "Design"
    RANDOM = "Random"
    SCALE = "Scale"
